/*
main source file
initialize event handlers, start animation thread
*/

import java.awt.*;
import java.awt.event.*;

public class BSP {
	public static void main(String[] args) {
		//create graphics window
		Figure f = new Figure("BSP Graphics Window");
		//add listeners for mouse events
		f.addWindowListener(new MyWindowAdapter());
		f.addMouseListener(new MyMouseAdapter());
		f.addMouseMotionListener(new MyMouseMotionAdapter());
		f.addKeyListener(new MyKeyAdapter());
		//start separate thread for animation
		Animation animation = new Animation(f);
		animation.start();		
	}
}

//handle window closing event
class MyWindowAdapter extends WindowAdapter {
	public void windowClosing(WindowEvent event) {
		System.exit(0);
	}
}

//handle mouse click events
class MyMouseAdapter extends MouseAdapter {
	public void mouseClicked(MouseEvent e) {
		Figure f = (Figure)e.getComponent();
		f.fill = !f.fill; //toggle fill switch
	}
}

//handle mouse motion events
class MyMouseMotionAdapter extends MouseMotionAdapter {
	public void mouseMoved(MouseEvent e) {
		Figure f = (Figure)e.getComponent();
		//mouse position determines velocity of rotation
		Point p = e.getPoint();
		f.velocity = new TwoDPoint(p.x-250,p.y-250);
	}
}

//handle key events
class MyKeyAdapter extends KeyAdapter {
	public void keyPressed(KeyEvent e) {
		Figure f = (Figure)e.getComponent();
		//uparrow key--move forward
		if (e.getKeyCode() == KeyEvent.VK_UP) {
			f.distanceVelocity = f.advance;
		}
		//downarrow key--move backward
		else if (e.getKeyCode() == KeyEvent.VK_DOWN) {
			f.distanceVelocity = -f.advance;
		}
	}
	//key released--stop zooming
	public void keyReleased(KeyEvent e) {
		Figure f = (Figure)e.getComponent();
		f.distanceVelocity = 0;
	}
}

//animation thread
//this thread will continuously render the image
class Animation extends Thread {
	Figure f;
	Animation(Figure f) {
		this.f = f;
	}		
	public void run() {
		f.animate(this);
	}
}

